
#property copyright "BJF Trading Group inc."
#property link      "http://iticsoftware.com"

#define major   2
#define minor   0

extern string _tmp1_ = " --- Trade parameters ---";
extern bool Acc5Digits = false;
extern bool Longs_On = true;
extern bool Shorts_On = true;
extern bool BrokerAllowsExpiration = true;
extern double Lots = 0.1;
extern double MaxLot = 33.0;
extern double ExtraLots = 0.1;
extern int TakeProfit = 50;
extern int Shift = 50;
extern int ExpHr = 4;
extern int Slippage = 3;
extern int Magic = 20090329;

extern string _tmp2_ = " --- Orders color ---";
extern color clBuy = DodgerBlue;
extern color clSell = Red;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#include <stdlib.mqh>
#include <stderror.mqh>

string ExpID = "MartingaleTheory_EURUSD_H1";

int StopLoss;

int MySlippage;
double MyPoint;

int BuyCnt, SellCnt;
int BuyStopCnt, SellStopCnt;

int RepeatN = 3;

void init() 
{
}

void deinit() 
{
}

void start() 
{
  //-----
  
  MyPoint = MarketInfo(Symbol(), MODE_POINT);
  MySlippage = Slippage;
  
  if (Acc5Digits)
  {
    MyPoint *= 10;
    MySlippage *= 10;
  }
  
  StopLoss = TakeProfit;
  
  //-----
  
  if (!BrokerAllowsExpiration)
  {
    CheckCustomExp();
  }
  
  //----

  RecountOrders();
  
  if (Longs_On && Shorts_On)
  {
    if (BuyStopCnt > 0 && SellStopCnt == 0)
    {
      BuyStopCnt = DeleteOrders(OP_BUYSTOP);
    }
  
    if (SellStopCnt > 0 && BuyStopCnt == 0)
    {
      SellStopCnt = DeleteOrders(OP_SELLSTOP);
    }
  }
  
  //-----
    
  RecountOrders();
    
  if (BuyCnt+SellCnt > 0) return;
  if (BuyStopCnt+SellStopCnt > 0) return;
  
  if (OrdersCountBar0() > 0) return;

  //-----

  double price, sl, tp;
  int ticket;
  datetime exp;
  
  double lot = GetLots();
  if (lot <= 0) return;

  if (Longs_On)
  {
    for (int i=0; i<RepeatN; i++)
    {
      RefreshRates();
      price = Ask + Shift*Point;
  
      sl = If(StopLoss > 0, price - StopLoss*Point, 0);
      tp = If(TakeProfit > 0, price + TakeProfit*Point, 0);
    
      exp = 0;
      if (BrokerAllowsExpiration)
      {
        exp = TimeCurrent() + ExpHr*60*60;
      }
        
      ticket = SetBuyStop(Symbol(), lot, price, sl, tp, Magic, ExpID, exp);
      if (ticket > 0) break;
    }
  }

  if (Shorts_On)
  {
    for (i=0; i<RepeatN; i++)
    {
      RefreshRates();
      price = Bid - Shift*Point;
    
      sl = If(StopLoss > 0, price + StopLoss*Point, 0);
      tp = If(TakeProfit > 0, price - TakeProfit*Point, 0);
    
      exp = 0;
      if (BrokerAllowsExpiration)
      {
        exp = TimeCurrent() + ExpHr*60*60;
      }

      ticket = SetSellStop(Symbol(), lot, price, sl, tp, Magic, ExpID, exp);
      if (ticket > 0) break;
    }
  }
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

double If(bool cond, double if_true, double if_false)
{
  if (cond) return (if_true);
  return (if_false);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

void RecountOrders()
{
  BuyCnt = 0;
  SellCnt = 0;
  BuyStopCnt = 0;
  SellStopCnt = 0;
  
  int cnt = OrdersTotal();
  for (int i=0; i < cnt; i++) 
  {
    if (!OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) continue;
    if (OrderSymbol() != Symbol()) continue;
    if (OrderMagicNumber() != Magic) continue;
    
    int type = OrderType();
    if (type == OP_BUY) BuyCnt++;
    if (type == OP_SELL) SellCnt++;
    if (type == OP_BUYSTOP) BuyStopCnt++;
    if (type == OP_SELLSTOP) SellStopCnt++;
  }
}

double GetLots() 
{
  if (LastOrderSelect(MODE_HISTORY, OP_BUY, OP_SELL))
  {  
    if (OrderClosePrice() == OrderStopLoss())
    {
      double lots = OrderLots();
      lots *= 2;
      if (DayOfWeek() != 5) lots += ExtraLots;
      
      if (lots > MaxLot) lots = MaxLot;
      return (lots);
    }
  }

  return (Lots);
}

void CheckCustomExp()
{
  int cnt = OrdersTotal();
  for (int i=cnt-1; i >= 0; i--) 
  {
    if (!OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) continue;
    if (OrderSymbol() != Symbol()) continue;
    if (OrderMagicNumber() != Magic) continue;
    
    if (TimeCurrent() - OrderOpenTime() < ExpHr*60*60) continue;
    
    int type = OrderType();
    if (type == OP_BUYSTOP || type == OP_SELLSTOP || type == OP_BUYLIMIT || type == OP_SELLLIMIT)
    {    
      DeleteOrder(OrderTicket());
    }
  }
}

bool LastOrderSelect(int pool, int type1, int type2 = -1)
{
  datetime tm = -1;
  int ticket = -1;

  if (pool == MODE_TRADES)
  {
    int cnt = OrdersTotal();
    for (int i=0; i < cnt; i++) 
    {
      if (!OrderSelect(i, SELECT_BY_POS, pool)) continue;
      if (OrderSymbol() != Symbol()) continue;
      if (OrderMagicNumber() != Magic) continue;
        
      int type = OrderType();
      if (type == type1 || type == type2)
      {
        if (OrderOpenTime() > tm)
        {
          tm = OrderOpenTime();
          ticket = OrderTicket();
        }
      }
    }
  
    return (OrderSelect(ticket, SELECT_BY_TICKET));
  }

  if (pool == MODE_HISTORY)
  {
    cnt = OrdersHistoryTotal();
    for (i=0; i < cnt; i++) 
    {
      if (!OrderSelect(i, SELECT_BY_POS, pool)) continue;
      if (OrderSymbol() != Symbol()) continue;
      if (OrderMagicNumber() != Magic) continue;
        
      type = OrderType();
      if (type == type1 || type == type2)
      {
        if (OrderCloseTime() > tm)
        {
          tm = OrderCloseTime();
          ticket = OrderTicket();
        }
      }
    }
  
    return (OrderSelect(ticket, SELECT_BY_TICKET));
  }
    
  return (false);
}

int OrdersCountBar0()
{
  int orders = 0;

  int cnt = OrdersTotal();
  for (int i=0; i<cnt; i++) 
  {
    if (!OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) continue;
    if (OrderSymbol() != Symbol()) continue;
    if (OrderMagicNumber() != Magic) continue;

    if (OrderOpenTime() >= Time[0]) orders++;
  }

  cnt = OrdersHistoryTotal();
  for (i=0; i<cnt; i++) 
  {
    if (!OrderSelect(i, SELECT_BY_POS, MODE_HISTORY)) continue;
    if (OrderSymbol() != Symbol()) continue;
    if (OrderMagicNumber() != Magic) continue;

    if (OrderOpenTime() >= Time[0]) orders++;
  }
 
  return (orders);
}

int DeleteOrders(int type) 
{
  int cnt = OrdersTotal();
  for (int i=cnt-1; i >= 0; i--) 
  {
    if (!OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) continue;
    if (OrderSymbol() != Symbol()) continue;
    if (OrderMagicNumber() != Magic) continue;
    
    if (OrderType() != type) continue;
    
    DeleteOrder(OrderTicket());
  }
  
  int orders = 0;
  cnt = OrdersTotal();
  for (i = 0; i < cnt; i++) 
  {
    if (!OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) continue;
    if (OrderSymbol() != Symbol()) continue;
    if (OrderMagicNumber() != Magic) continue;
    
    if (OrderType() == type) orders++;
  }
  
  return (orders); 
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

int SleepOk = 2000;
int SleepErr = 6000;

int SetBuyStop(string symbol, double lot, double price, double sl, double tp, int magic, string comment, datetime exp) 
{
  int dig = MarketInfo(symbol, MODE_DIGITS);

  price = NormalizeDouble(price, dig);
  sl = NormalizeDouble(sl, dig);
  tp = NormalizeDouble(tp, dig);
  
  string _lot = DoubleToStr(lot, 2);
  string _price = DoubleToStr(price, dig);
  string _sl = DoubleToStr(sl, dig);
  string _tp = DoubleToStr(tp, dig);

  Print("SetBuyStop \"", symbol, "\", ", _lot, ", ", _price, ", ", MySlippage, ", ", _sl, ", ", _tp, ", ", magic, ", \"", comment, "\"");
  
  int res = OrderSend(symbol, OP_BUYSTOP, lot, price, MySlippage, sl, tp, comment, magic, exp, clBuy);
  if (res >= 0) {
    Sleep(SleepOk);
    return (res);
  } 	
   	
  int code = GetLastError();
  Print("Error setting BUYSTOP order: ", ErrorDescription(code), " (", code, ")");
  Sleep(SleepErr);
	
  return (-1);
}

int SetSellStop(string symbol, double lot, double price, double sl, double tp, int magic, string comment, datetime exp) 
{
  int dig = MarketInfo(symbol, MODE_DIGITS);

  price = NormalizeDouble(price, dig);
  sl = NormalizeDouble(sl, dig);
  tp = NormalizeDouble(tp, dig);
  
  string _lot = DoubleToStr(lot, 2);
  string _price = DoubleToStr(price, dig);
  string _sl = DoubleToStr(sl, dig);
  string _tp = DoubleToStr(tp, dig);

  Print("SetSellStop \"", symbol, "\", ", _lot, ", ", _price, ", ", MySlippage, ", ", _sl, ", ", _tp, ", ", magic, ", \"", comment, "\"");
  
  int res = OrderSend(symbol, OP_SELLSTOP, lot, price, MySlippage, sl, tp, comment, magic, exp, clSell);
  if (res >= 0) {
    Sleep(SleepOk);
    return (res);
  } 	
   	
  int code = GetLastError();
  Print("Error setting SELLSTOP order: ", ErrorDescription(code), " (", code, ")");
  Sleep(SleepErr);
	
  return (-1);
}

bool DeleteOrder(int ticket) 
{
  Print("DeleteOrder ", ticket);
  
  bool res = OrderDelete(ticket);
  if (res) {
    Sleep(SleepOk);
    return (res);
  } 	
   	
  int code = GetLastError();
  Print("DeleteOrder failed: ", ErrorDescription(code), " (", code, ")");
  Sleep(SleepErr);
	
  return (false);
}

